@@ -2,7 +2,7 @@
  * libdvdcss.c: DVD reading library.
  *****************************************************************************
  * Copyright (C) 1998-2001 VideoLAN
- * $Id: libdvdcss.c,v 1.6 2002/04/04 23:44:20 gbazin Exp $
+ * $Id: libdvdcss.c,v 1.9 2002/05/16 20:40:54 hjort Exp $
  *
  * Authors: Stphane Borel <stef@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -71,6 +71,7 @@ struct iovec
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
+static int _dvdcss_use_ioctls ( dvdcss_handle dvdcss );
 static int _dvdcss_open  ( dvdcss_handle, char *psz_target );
 static int _dvdcss_close ( dvdcss_handle );
 static int _dvdcss_readv ( dvdcss_handle, struct iovec *p_iovec, int i_blocks );
@@ -162,7 +163,7 @@ extern dvdcss_handle dvdcss_open ( char *psz_target )
     /* Initialize structure with default values */
     dvdcss->p_titles = NULL;
     dvdcss->psz_error = "no error";
-    dvdcss->i_method = DVDCSS_METHOD_TITLE;
+    dvdcss->i_method = DVDCSS_METHOD_KEY;
     dvdcss->b_debug = 0;
     dvdcss->b_errors = 1;
 
@@ -219,19 +220,22 @@ extern dvdcss_handle dvdcss_open ( char *psz_target )
         free( dvdcss );
         return NULL;
     }
+    
+    dvdcss->b_encrypted = 1; /* Assume the worst */
+    dvdcss->b_ioctls = _dvdcss_use_ioctls( dvdcss );
 
-    i_ret = CSSTest( dvdcss );
-    if( i_ret < 0 )
+    if( dvdcss->b_ioctls )
     {
-        _dvdcss_error( dvdcss, "CSS test failed" );
-        /* Disable the CSS ioctls and hope that it works? */
-        dvdcss->b_ioctls = 0;
-        dvdcss->b_encrypted = 1;
-    }
-    else
-    {
-        dvdcss->b_ioctls = 1;
-        dvdcss->b_encrypted = i_ret;
+        i_ret = CSSTest( dvdcss );
+	if( i_ret < 0 )
+	{
+	    /* Disable the CSS ioctls and hope that it works? */
+	    dvdcss->b_ioctls = 0;
+	}
+	else
+	{
+	    dvdcss->b_encrypted = i_ret;
+	}
     }
 
     /* If disc is CSS protected and the ioctls work, authenticate the drive */
@@ -386,31 +390,33 @@ extern int dvdcss_read ( dvdcss_handle dvdcss, void *p_buffer,
         return i_ret;
     }
 
-    /* For what we believe is an unencrypted title, 
-       check that there are no encrypted blocks */
     if( !( Pkey[0] | Pkey[1] | Pkey[2] | Pkey[3] | Pkey[4] ) ) 
     {
+        /* For what we believe is an unencrypted title, 
+	 * check that there are no encrypted blocks */
         for( i_index = i_ret; i_index; i_index-- )
         {
             if( ((u8*)p_buffer)[0x14] & 0x30 )
             {
                 _dvdcss_error( dvdcss, "no key but found encrypted block" );
                 /* Only return the initial range of unscrambled blocks? */
-                i_ret = i_index;
                 /* or fail completely? return 0; */
+		break;
             }
             (u8*)p_buffer += DVDCSS_BLOCK_SIZE;
         }
     }
-
-    /* Decrypt the blocks we managed to read */
-    for( i_index = i_ret; i_index; i_index-- )
+    else 
     {
-        CSSDescrambleSector( dvdcss->css.p_title_key, p_buffer );
-        ((u8*)p_buffer)[0x14] &= 0x8f;
-        (u8*)p_buffer += DVDCSS_BLOCK_SIZE;
+        /* Decrypt the blocks we managed to read */
+        for( i_index = i_ret; i_index; i_index-- )
+	{
+	    CSSDescrambleSector( dvdcss->css.p_title_key, p_buffer );
+	    ((u8*)p_buffer)[0x14] &= 0x8f;
+	    (u8*)p_buffer += DVDCSS_BLOCK_SIZE;
+	}
     }
-
+    
     return i_ret;
 }
 
@@ -499,6 +505,54 @@ extern int dvdcss_close ( dvdcss_handle dvdcss )
 
 /* Following functions are local */
 
+static int _dvdcss_use_ioctls( dvdcss_handle dvdcss )
+{
+#if defined( WIN32 )
+    /* Some one need to implement this for Windows */
+    if( WIN2K )
+    {
+	return 1;	
+    }
+    else
+    {
+	return 1;	
+    }
+#else
+    struct stat fileinfo;
+    int ret;
+
+    ret = fstat( dvdcss->i_fd, &fileinfo );
+    if( ret < 0 )
+    {
+	return 1;  /* What to do?  Be conservative and try to use the ioctsl */
+    }
+    
+    /* Complete this list and check that we test for the right things 
+     * (I've assumed for all OSs that 'r', (raw) device, are char devices
+     *  and those that don't contain/use an 'r' in the name are block devices)
+     *
+     * Linux    needs a block device
+     * Solaris  needs a char device
+     * Darwin   needs a char device 
+     * OpenBSD  needs a char device
+     * NetBSD   needs a char device
+     * FreeBSD  can use either the block or the char device
+     * BSD/OS   can use either the block or the char device
+     */
+    
+    /* Check if this is a block/char device */
+    if( S_ISBLK( fileinfo.st_mode ) || 
+	S_ISCHR( fileinfo.st_mode ) )
+    {
+	return 1;
+    }
+    else
+    {
+	return 0;
+    }
+#endif
+}
+
 static int _dvdcss_open ( dvdcss_handle dvdcss, char *psz_target )
 {
 #if defined( WIN32 )
@@ -678,7 +732,9 @@ int _dvdcss_read ( dvdcss_handle dvdcss, void *p_buffer, int i_blocks )
 
 #else
     int i_bytes;
-
+    /* TODO: !!! errors are mangled and disarded (-1/2048 = 0)
+     * and partial reads are wrong,i.e 2200/2048 = 1 
+     * but the location has advanced 2200 bytes (lseek possition that is) */
     i_bytes = read( dvdcss->i_read_fd, p_buffer,
                     (size_t)i_blocks * DVDCSS_BLOCK_SIZE );
     return i_bytes / DVDCSS_BLOCK_SIZE;
